<form action="{% url 'kalabash_sievefilters:fs_save' name %}" 
      method="POST" id="feditor" name="feditor">{% csrf_token %}
  <textarea id="scriptcontent" name="scriptcontent">{{ scriptcontent }}</textarea>
</form>
