{% load i18n sfilters_tags %}
{% if fs.filters %}
<table class="table table-striped table-bordered">
  <tr>
    <th width="10%">{% trans "Actions" %}</th>
    <th>{% trans "Name" %}</th>
    <th width="5%">{% trans "Active" %}</th>
  </tr>
  {% for filter in fs.filters %}
  <tr>
    <td>{% filter_actions fs.name filter forloop.counter forloop.last %}</td>
    <td>{{ filter.name }}</td>
    <td>
      <a name="togglestate" 
         class={% if filter.enabled %}"green"{% else %}"red"{% endif %}
         href="{% url 'kalabash_sievefilters:filter_toggle_state' fs.name filter.name %}">
        {{ filter.enabled|yesno }}
      </a>
    </td>
  </tr>
  {% endfor %}
</table>
{% else %}
<div class="alert alert-info">{% trans "No filter defined yet" %}</div>
{% endif %}
